// Copyright (c) Mito
import React, { useEffect, useState } from 'react';
// import css
import '../../../css/graph-sidebar/graph-sidebar.css';
import '../../../css/graph-sidebar/loading-spinner.css';
import DownloadIcon from '../icons/DownloadIcon';
import XIcon from '../icons/XIcon';
import GraphAxis, { GraphAxisType } from './GraphAxis';
import GraphColumnSelector from './GraphColumnSelector';
import LoadingSpinner from './LoadingSpinner';
/*
    This is the main component that displays all graphing
    functionality, allowing the user to build and view graphs.
*/
const GraphSidebar = (props) => {
    const [xAxis, setXAxis] = useState(undefined);
    const [sheetIndexXAxis, setSheetIndexXAxis] = useState(undefined);
    const [yAxis, setYAxis] = useState(undefined);
    const [sheetIndexYAxis, setSheetIndexYAxis] = useState(undefined);
    // We store each column selector seperatly b/c of race conditions around
    // closing the column selector when clicking off of it
    const [xAxisColumnSelectorOpen, setXAxisColumnSelectorOpen] = useState(false);
    const [yAxisColumnSelectorOpen, setYAxisColumnSelectorOpen] = useState(false);
    const [base64PNGImage, setBase64PNGImage] = useState('');
    const [alt, setAlt] = useState('');
    const [loading, setLoading] = useState(false);
    // We log when the graph has been opened
    useEffect(() => {
        void props.mitoAPI.sendLogMessage('opened_graph');
    }, []);
    /*
        Create a new graph each time the x and y axises change, unless they are both blank.
        We do this in the useEffect for two reasons:
            1. It makes sure we don't have to worry about race conditions between setting the useState and calling getGraph
            2. It protects us from incorrect settings of the x and y axis that don't use the setAxis function.
    */
    useEffect(() => {
        const getGraphAsync = async () => {
            setLoading(true);
            const loadedBase64PNGImage = await props.mitoAPI.getGraph(xAxis, sheetIndexXAxis, yAxis, sheetIndexYAxis);
            if (loadedBase64PNGImage === '') {
                setAlt('Sorry, we were unable to create your graph.');
            }
            else {
                setBase64PNGImage(loadedBase64PNGImage);
            }
            setLoading(false);
        };
        // If a change to the x or y axis is made and there is at least an x axis, then get the graph
        if (xAxis !== undefined) {
            void getGraphAsync();
        }
        else {
            // Otherwise, the user may have unset the x axis, so we make sure to clear the graph
            setBase64PNGImage('');
        }
        // We also log when the columns are selected, when they change
        void props.mitoAPI.sendLogMessage('graph_selected_column_changed', {
            'generated_graph': xAxis !== undefined,
            'column_header_x_axis': xAxis,
            'sheet_index_x_axis': sheetIndexXAxis,
            'column_header_y_axis': yAxis,
            'sheet_index_y_axis': sheetIndexYAxis,
        });
    }, [xAxis, yAxis]);
    const setAxis = async (axis, columnHeader) => {
        var _a, _b;
        if (axis === GraphAxisType.X_AXIS) {
            setSheetIndexXAxis(props.selectedSheetIndex);
            setXAxis(columnHeader);
        }
        else if (axis === GraphAxisType.Y_AXIS) {
            setSheetIndexYAxis(props.selectedSheetIndex);
            setYAxis(columnHeader);
        }
        props.setSelectedColumn(columnHeader);
        // Scroll the selected column into view
        (_b = (_a = window.gridApiMap) === null || _a === void 0 ? void 0 : _a.get(props.model_id)) === null || _b === void 0 ? void 0 : _b.ensureColumnVisible(columnHeader);
    };
    const displayBlankAxisesStyle = base64PNGImage === '' ? 'display-blank-axises' : '';
    if (props.sheetJSONArray.length === 0) {
        return (React.createElement("div", { className: 'graph-sidebar-div' },
            React.createElement("div", { className: 'graph-sidebar-no-data-div' },
                React.createElement("p", { className: 'graph-sidebar-no-data-text' }, "You need to import data into Mito before graphing!"))));
    }
    else {
        return (React.createElement("div", { className: 'graph-sidebar-div' },
            React.createElement("table", { className: 'graph-sidebar-table' },
                React.createElement("tbody", null,
                    React.createElement("tr", { style: { height: '85%' } },
                        React.createElement("td", { style: { width: '5%' } },
                            React.createElement(GraphAxis, { axis: GraphAxisType.Y_AXIS, columnHeader: yAxis, openColumnSelector: () => setYAxisColumnSelectorOpen(true) })),
                        React.createElement("td", { className: displayBlankAxisesStyle, style: { width: '85%' } },
                            React.createElement("div", { className: 'graph-sidebar-table-graph-cell' },
                                React.createElement("img", { key: base64PNGImage, className: 'graph-sidebar-table-graph', src: 'data:image/png;base64, ' + base64PNGImage, alt: alt }))),
                        React.createElement("td", { className: 'graph-sidebar-toolbar-div' },
                            React.createElement("div", { onClick: () => {
                                    props.setGraphSidebarOpen(false);
                                    void props.mitoAPI.sendLogMessage('closed_graph');
                                } },
                                React.createElement(XIcon, null)),
                            React.createElement("div", { className: 'mt-2', 
                                // We do a little styling to let the user know the button is unclickable
                                style: { opacity: xAxis === undefined ? '50%' : '100%' }, onClick: async () => {
                                    // We don't try and download if there's nothing to download!
                                    if (xAxis === undefined) {
                                        return;
                                    }
                                    const data = await props.mitoAPI.getGraph(xAxis, sheetIndexXAxis, yAxis, sheetIndexYAxis, true);
                                    // Download the graph by generating an a tag
                                    // Download pattern found: https://stackoverflow.com/questions/3916191/download-data-url-file
                                    const link = document.createElement("a");
                                    link.download = 'mito-graph-download.png';
                                    link.href = 'data:image/png;base64, ' + data;
                                    document.body.appendChild(link);
                                    link.click();
                                    // Delete the a tag from the document
                                    document.body.removeChild(link);
                                    // We log the download of the graph
                                    void props.mitoAPI.sendLogMessage('click_download_graph', {
                                        'generated_graph': xAxis !== undefined,
                                        'column_header_x_axis': xAxis,
                                        'sheet_index_x_axis': sheetIndexXAxis,
                                        'column_header_y_axis': yAxis,
                                        'sheet_index_y_axis': sheetIndexYAxis,
                                    });
                                } },
                                React.createElement(DownloadIcon, null)))),
                    React.createElement("tr", { style: { height: '20%' } },
                        React.createElement("td", null),
                        React.createElement("td", null,
                            React.createElement("div", { className: 'graph-sidebar-x-axis-div' },
                                React.createElement(GraphAxis, { axis: GraphAxisType.X_AXIS, columnHeader: xAxis, openColumnSelector: () => setXAxisColumnSelectorOpen(true) }))),
                        React.createElement("td", null)))),
            xAxisColumnSelectorOpen &&
                React.createElement(GraphColumnSelector, { axis: GraphAxisType.X_AXIS, selectedColumnHeader: xAxis, columnHeaders: props.sheetJSONArray[props.selectedSheetIndex].columns, columnTypes: props.columnTypes, setAxis: setAxis, closeColumnSelector: () => setXAxisColumnSelectorOpen(false) }),
            !xAxisColumnSelectorOpen && yAxisColumnSelectorOpen &&
                React.createElement(GraphColumnSelector, { axis: GraphAxisType.Y_AXIS, selectedColumnHeader: yAxis, columnHeaders: props.sheetJSONArray[props.selectedSheetIndex].columns, columnTypes: props.columnTypes, setAxis: setAxis, closeColumnSelector: () => setYAxisColumnSelectorOpen(false) }),
            loading &&
                React.createElement("div", { className: 'loading-spinner-div' },
                    React.createElement(LoadingSpinner, null),
                    React.createElement("p", { className: 'loading-spinner-div-text' }, "loading"))));
    }
};
export default GraphSidebar;
//# sourceMappingURL=GraphSidebar.js.map